////////////////////////////////////////////////////////////////////////////////////
//StelephonyApi.h - declarations of function, structures and variables
//				exported by the Stelephony.dll. 
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//
// Versions:
//	v 1,0,0,1	May  29	2008	David Rokhvarg	Initial Version
//		Implemented FSK Caller ID detection for Analog FXO.
//
//	v 1,0,0,2	June 12	2008	David Rokhvarg
//		Software DTMF detection
//
//	v 1,0,0,3	July 30	2008	Konrad Hammel
//		Q931 event detection
////////////////////////////////////////////////////////////////////////////////////
#pragma once

#ifdef STELEPHONY_EXPORTS
# define STELEPHONY_API __declspec(dllexport)
#else
# define STELEPHONY_API __declspec(dllimport)
#endif

typedef enum _stelephony_status{

	STEL_STATUS_SUCCESS=0,
	STEL_STATUS_MEMORY_ALLOCATION_ERROR,
	STEL_STATUS_INVALID_INPUT_ERROR,
	STEL_STATUS_REQUEST_INVALID_FOR_CURRENT_OBJECT_STATE_ERROR,
	STEL_STATUS_DECODER_LIB_CO_CREATE_INSTANCE_REGDB_E_CLASSNOTREG_ERROR,
	STEL_STATUS_DECODER_LIB_CO_CREATE_INSTANCE_CLASS_E_NOAGGREGATION_ERROR,
	STEL_STATUS_DECODER_LIB_CO_CREATE_INSTANCE_E_NOINTERFACE_ERROR,
	STEL_STATUS_DECODER_LIB_CO_CREATE_INSTANCE_UNKNOWN_ERROR,
	STEL_STATUS_DECODER_LIB_QUERY_INTERFACE_ERROR,
	STEL_STATUS_DECODER_LIB_FIND_CONNECTION_POINT_ERROR,
	STEL_STATUS_DECODER_LIB_MEMORY_ALLOCATION_ERROR,
	STEL_STATUS_INVALID_EVENT_ERROR,
	STEL_STATUS_INVALID,_EVENT_CONTROL_CODE_ERROR,
	STEL_STATUS_OBJECT_DISABLED_ERROR

}stelephony_status_t;

typedef enum _stelephony_event{

	STEL_EVENT_FSK_CALLER_ID=1,
	STEL_EVENT_DTMF,
	STEL_EVENT_Q931
}stelephony_event_t;

typedef enum _stelephony_control_code{

	STEL_CTRL_CODE_ENABLE=1,
	STEL_CTRL_CODE_DISABLE

}stelephony_control_code_t;

typedef enum _stelephony_option{

	STEL_OPTION_MULAW=1,
	STEL_OPTION_ALAW

}stelephony_option_t;

typedef struct {
	int				len_callRef;						//Length of call reference field
	char			callRef[6];							//call reference value, hex
	char			msg_type[25];						//Null terminated string containing the Q931 message type
	int				chan;								//b-channel call we be using
	int				called_num_digits_count;			//Number of Digits in the called number
	char			called_num_digits[32];				//Null terminated string containing the called number
	int				calling_num_digits_count;			//Number of digits in the calling number
	char			calling_num_digits[32];				//Null terminated string containing the calling number
	int				cause_code;							//the release cause code
	SYSTEMTIME		tv;									//time value that message was received by Q931 decoder
	int				calling_num_screening_ind;
	int				calling_num_presentation;
	int				rdnis_digits_count;					//Number of RDNIS digits found
	char			rdnis_string[81];					//RDNIS value
	unsigned char	data[8188];							//the entire data
	int		dataLength;									//the length of the data

}stelephony_q931_event;

typedef struct{
	void	(*FSKCallerIDEvent)(void *callback_context, LPCTSTR Name, LPCTSTR CallerNumber, LPCTSTR CalledNumber, LPCTSTR DateTime);
	void	(*DTMFEvent)(void *callback_context, long Key);
	void	(*Q931Event)(void *callback_context, stelephony_q931_event *pEvent);
}stelephony_callback_functions_t;

STELEPHONY_API stelephony_status_t StelSetup(void **stelObj, void *callback_context, stelephony_callback_functions_t* scf);
STELEPHONY_API stelephony_status_t StelCleanup(void *stelObj);
STELEPHONY_API stelephony_status_t StelStreamInput(void *stelObj, void *data, int dataLength);
STELEPHONY_API stelephony_status_t StelEventControl(void *stelObjPtr, stelephony_event_t Event, stelephony_control_code_t ControlCode, void *optionalData);




