/***********************************************************************************
* sang_status_defines.h - Definitions of cross-platform return codes.
*							
*
* Author	:	David Rokhvarg	<davidr@sangoma.com>
*
* Copyright	:	(c) 2000 Sangoma Technologies Inc.
*
************************************************************************************/

#ifndef _SANG_STATUS_DEFINES_H_
#define _SANG_STATUS_DEFINES_H_

/*
	Extra care should be taken when changing SANG_STATUS_xxx definitions
	to keep compatibility with existing binary files!
*/
typedef enum SANG_STATUS{

	//An operation completed successfully
	SANG_STATUS_SUCCESS=0,
	//Two threads attempting to access device at the same time.
	SANG_STATUS_COMMAND_ALREADY_RUNNING,
	//Buffer passed to API is too small.
	SANG_STATUS_BUFFER_TOO_SMALL,
	//Kernel error
	SANG_STATUS_FAILED_TO_LOCK_USER_MEMORY,
	//Kernel error
	SANG_STATUS_FAILED_ALLOCATE_MEMORY,
	//Command is invalid for device type.
	SANG_STATUS_INVALID_DEVICE_REQUEST,
	//General 
	SANG_STATUS_INVALID_PARAMETER,

	//For internal API use.
	SANG_STATUS_DATA_QUEUE_EMPTY,
	SANG_STATUS_DATA_QUEUE_FULL,

	//
	//Return codes specific for data reception:
	//
	//No data was received. NOT an error.
	SANG_STATUS_RX_DATA_TIMEOUT,
	//Data was received.
	SANG_STATUS_RX_DATA_AVAILABLE,

	//
	//Return codes specific for data transmission:
	//
	//Transmit command timed out.
	SANG_STATUS_TX_TIMEOUT,
	//Longer than MTU
	SANG_STATUS_TX_DATA_TOO_LONG,
	//Shorter than minimum
	SANG_STATUS_TX_DATA_TOO_SHORT,
	//For internal API use.
	SANG_STATUS_RESERVED_1,

	//Physical line (T1/E1/Serial) is disconnected.
	SANG_STATUS_LINE_DISCONNECTED,

	//The Communication Protocol (CHDLC/Frame Relay/PPP/LAPD...) is disconnected.
	SANG_STATUS_PROTOCOL_DISCONNECTED,

	SANG_STATUS_GENERAL_ERROR,
	SANG_STATUS_DEVICE_BUSY,
	SANG_STATUS_INVALID_DEVICE,
	SANG_STATUS_IO_ERROR,
	SANG_STATUS_UNSUPPORTED_FUNCTION,
	SANG_STATUS_UNSUPPORTED_PROTOCOL,
	SANG_STATUS_DEVICE_ALREADY_EXIST,
	SANG_STATUS_DEV_INIT_INCOMPLETE,
	SANG_STATUS_TRACE_QUEUE_EMPTY,
	SANG_STATUS_OPTION_NOT_SUPPORTED,

	//TDM Voice API definitions
	SANG_STATUS_TDM_EVENT_AVAILABLE,

	//Physical line (T1/E1/Serial) is connected.
	//(has to be here to keep binary compatibility)
	SANG_STATUS_LINE_CONNECTED,
	//The Communication Protocol (CHDLC/Frame Relay/PPP/LAPD...) is connected.
	//(has to be here to keep binary compatibility)
	SANG_STATUS_PROTOCOL_CONNECTED,

	SANG_STATUS_REGISTRY_ERROR,
	SANG_STATUS_CAN_NOT_STOP_DEVICE_WHEN_ALREADY_STOPPED,
	SANG_STATUS_CAN_NOT_RUN_TWO_PORT_CMDS_AT_THE_SAME_TIME,
	SANG_STATUS_ASSOCIATED_IRP_SYSTEM_BUFFER_NULL_ERROR,
	//This error caused by mismatch between structures used by application and
	//the API driver. Solution is to compile application with header files from
	//the currently running API driver package.
	SANG_STATUS_STRUCTURE_SIZE_MISMATCH_ERROR
}SANG_STATUS_T;


#define SDLA_DECODE_SANG_STATUS(status)	\
(status == SANG_STATUS_SUCCESS) ? "SANG_STATUS_SUCCESS" :\
(status == SANG_STATUS_COMMAND_ALREADY_RUNNING) ? "SANG_STATUS_COMMAND_ALREADY_RUNNING":\
(status == SANG_STATUS_BUFFER_TOO_SMALL) ? "SANG_STATUS_BUFFER_TOO_SMALL":\
(status == SANG_STATUS_FAILED_TO_LOCK_USER_MEMORY) ? "SANG_STATUS_FAILED_TO_LOCK_USER_MEMORY":\
(status == SANG_STATUS_FAILED_ALLOCATE_MEMORY) ? "SANG_STATUS_FAILED_ALLOCATE_MEMORY":\
(status == SANG_STATUS_INVALID_DEVICE_REQUEST) ? "SANG_STATUS_INVALID_DEVICE_REQUEST":\
(status == SANG_STATUS_INVALID_PARAMETER) ? "SANG_STATUS_INVALID_PARAMETER":\
(status == SANG_STATUS_DATA_QUEUE_EMPTY) ? "SANG_STATUS_DATA_QUEUE_EMPTY":\
(status == SANG_STATUS_DATA_QUEUE_FULL) ? "SANG_STATUS_DATA_QUEUE_FULL":\
(status == SANG_STATUS_RX_DATA_TIMEOUT) ? "SANG_STATUS_RX_DATA_TIMEOUT":\
(status == SANG_STATUS_RX_DATA_AVAILABLE) ? "SANG_STATUS_RX_DATA_AVAILABLE":\
(status == SANG_STATUS_TX_TIMEOUT) ? "SANG_STATUS_TX_TIMEOUT":\
(status == SANG_STATUS_TX_DATA_TOO_LONG) ? "SANG_STATUS_TX_DATA_TOO_LONG":\
(status == SANG_STATUS_TX_DATA_TOO_SHORT) ? "SANG_STATUS_TX_DATA_TOO_SHORT":\
(status == SANG_STATUS_LINE_DISCONNECTED) ? "SANG_STATUS_LINE_DISCONNECTED":\
(status == SANG_STATUS_LINE_CONNECTED) ? "SANG_STATUS_LINE_CONNECTED":\
(status == SANG_STATUS_PROTOCOL_DISCONNECTED) ? "SANG_STATUS_PROTOCOL_DISCONNECTED":\
(status == SANG_STATUS_PROTOCOL_CONNECTED) ? "SANG_STATUS_PROTOCOL_CONNECTED":\
(status == SANG_STATUS_GENERAL_ERROR) ? "SANG_STATUS_GENERAL_ERROR":\
(status == SANG_STATUS_DEVICE_BUSY) ? "SANG_STATUS_DEVICE_BUSY":\
(status == SANG_STATUS_INVALID_DEVICE) ? "SANG_STATUS_INVALID_DEVICE":\
(status == SANG_STATUS_IO_ERROR) ? "SANG_STATUS_IO_ERROR":\
(status == SANG_STATUS_UNSUPPORTED_FUNCTION) ? "SANG_STATUS_UNSUPPORTED_FUNCTION":\
(status == SANG_STATUS_UNSUPPORTED_PROTOCOL) ? "SANG_STATUS_UNSUPPORTED_PROTOCOL":\
(status == SANG_STATUS_DEVICE_ALREADY_EXIST) ? "SANG_STATUS_DEVICE_ALREADY_EXIST":\
(status == SANG_STATUS_DEV_INIT_INCOMPLETE) ? "SANG_STATUS_DEV_INIT_INCOMPLETE":\
(status == SANG_STATUS_TDM_EVENT_AVAILABLE) ? "SANG_STATUS_TDM_EVENT_AVAILABLE":\
(status == SANG_STATUS_REGISTRY_ERROR) ? "SANG_STATUS_REGISTRY_ERROR":\
(status == SANG_STATUS_CAN_NOT_STOP_DEVICE_WHEN_ALREADY_STOPPED) ? "SANG_STATUS_CAN_NOT_STOP_DEVICE_WHEN_ALREADY_STOPPED":\
(status == SANG_STATUS_CAN_NOT_RUN_TWO_PORT_CMDS_AT_THE_SAME_TIME) ? "SANG_STATUS_CAN_NOT_RUN_TWO_PORT_CMDS_AT_THE_SAME_TIME":\
(status == SANG_STATUS_ASSOCIATED_IRP_SYSTEM_BUFFER_NULL_ERROR) ? "SANG_STATUS_ASSOCIATED_IRP_SYSTEM_BUFFER_NULL_ERROR":\
(status == SANG_STATUS_STRUCTURE_SIZE_MISMATCH_ERROR) ? "SANG_STATUS_STRUCTURE_SIZE_MISMATCH_ERROR":\
"Status Unknown"


#if defined(WAN_KERNEL) || defined(BUILD_WANEC_CLIENT)
/*
	Cross-Platform return codes.
	SOME of these return codes are defined in errno.h, but not ALL of them.
*/
# undef EFAULT
# undef EBUSY
# undef ENODEV
# undef EINVAL
# undef EIO
# undef EPFNOSUPPORT
# undef EPROTONOSUPPORT
# undef ENOMEM
# undef EEXIST
# undef ENOBUFS
# undef EOPNOTSUPP
# undef ENXIO

# define EFAULT			SANG_STATUS_GENERAL_ERROR
# define EBUSY			SANG_STATUS_DEVICE_BUSY
# define ENODEV			SANG_STATUS_INVALID_DEVICE
# define EINVAL			SANG_STATUS_INVALID_PARAMETER
# define EIO			SANG_STATUS_IO_ERROR
# define EPFNOSUPPORT		SANG_STATUS_UNSUPPORTED_FUNCTION
# define EPROTONOSUPPORT	SANG_STATUS_UNSUPPORTED_PROTOCOL
# define ENOMEM			SANG_STATUS_FAILED_ALLOCATE_MEMORY
# define EEXIST			SANG_STATUS_DEVICE_ALREADY_EXIST
# define ENOBUFS		ENOMEM
# define EOPNOTSUPP		SANG_STATUS_OPTION_NOT_SUPPORTED
# define ENXIO			EFAULT
# define ENETDOWN		SANG_STATUS_LINE_DISCONNECTED
#endif

#endif //_SANG_STATUS_DEFINES_H_
